/*testcaput.c*/
/* performCaput.c,v 1.1 2005/11/04 16:11:51 xusf Exp */
/***********************************************************************
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory, and the Regents of the University of
* California, as Operator of Los Alamos National Laboratory, and
* Berliner Elektronenspeicherring-Gesellschaft m.b.H. (BESSY).
* asynDriver is distributed subject to a Software License Agreement
* found in file LICENSE that is included with this distribution.
***********************************************************************/
/*
 * Author: Marty Kraimer
 * Date:   03NOV2005
 */

#include <stddef.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "epicsTime.h"
#include "cadef.h"

epicsTimeStamp time_now;
char myvalue[20];
static int stateChanges = 0;
static int readAccess = 0;
static int writeAccess = 0;
static int accessChanged = 0;
static int newvalue = 0;

typedef struct
{
    chid mychid;
    short read_access;
    short write_access;
} MYNODE;

MYNODE *pmynode = NULL;

void accessRightsCallback (struct access_rights_handler_args arha)
{
    chid chid = arha.chid;
    MYNODE *pnode = (MYNODE *) ca_puser (chid);

    if (ca_read_access (chid) != pnode->read_access)
      {
	  if (ca_read_access (chid))
	      readAccess++;
	  else
	      readAccess--;
	  pnode->read_access = ca_read_access (chid);
      }
    if (ca_write_access (chid) != pnode->write_access)
      {
	  if (ca_write_access (chid))
	      writeAccess++;
	  else
	      writeAccess--;
	  pnode->write_access = ca_write_access (chid);
      }
    accessChanged = 1;
}

void connection_callback (struct connection_handler_args cha)
{
    stateChanges++;
}

int main (int argc, char **argv)
{
    int npv;
    int status;
    int i;
    int notConnected;
    int prevNotConnected;
    char timestamp[40];
    double pendEventTime = 0;

    if (argc != 3)
      {
	  printf ("usage: testcaput <nrecords> <pendEventTime>\n");
	  exit (1);
      }
    sscanf (argv[1], "%d", &npv);
    sscanf (argv[2], "%lf", &pendEventTime);
    pmynode = (MYNODE *) calloc (npv, sizeof (MYNODE));
    notConnected = npv + 1;	/*Make message get printed first time */
    for (i = 0; i < npv; i++)
      {
	  memset ((char *) &pmynode[i], 0, sizeof (MYNODE));
      }
    SEVCHK (ca_task_initialize (), "ca_task_initialize");
    for (i = 0; i < npv; i++)
      {
	  char name[40];

	  sprintf (name, "xusf%04d", i);
	  SEVCHK (ca_search_and_connect (name, &pmynode[i].mychid,
					 connection_callback, &pmynode[i]),
		  "ca_search_and_connect");

	  SEVCHK (ca_replace_access_rights_event (pmynode[i].mychid,
						  accessRightsCallback),
		  "ca_replace_access_rights_event");
      }
    while (1)
      {
	  newvalue++;
	  sprintf (myvalue, "%d", newvalue);
	  if (newvalue >= 10)
	      newvalue = 0;
	  status = ca_pend_event (pendEventTime);
	  if (status != ECA_NORMAL && status != ECA_TIMEOUT)
	      SEVCHK (status, "ca_pend_event");
	  if (epicsTimeGetCurrent (&time_now) != 0)
	      printf ("epicsTimeGetCurrent error in main\n");
	  epicsTimeToStrftime (timestamp, sizeof (timestamp),
			       "%Y/%m/%d %H:%M:%S.%06f", &time_now);
	  prevNotConnected = notConnected;
	  notConnected = 0;
	  for (i = 0; i < npv; i++)
	    {
		if (ca_state (pmynode[i].mychid) != cs_conn)
		  {
		      notConnected++;
		      continue;
		  }
		status = ca_put (DBR_STRING, pmynode[i].mychid, myvalue);
		if (status != ECA_NORMAL)
		    printf ("%s From ca_put\n", ca_message (status));
	    }
	  SEVCHK (ca_flush_io (), "ca_flush_io");;
	  if (accessChanged)
	    {
		printf ("readAccess %5.5d writeAccess %5.5d time %s\n",
			readAccess, writeAccess, timestamp);
		accessChanged = 0;
	    }
	  if (prevNotConnected != notConnected)
	    {
		printf ("notConnected %5.5d time %s\n", notConnected,
			timestamp);
	    }
      }
    return (0);
}
