/* rtemsSampler.c */
/* rtemsSampler.c,v 1.2 2005/12/08 15:24:38 xusf Exp */
/***********************************************************************
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory, and the Regents of the University of
* California, as Operator of Los Alamos National Laboratory, and
* Berliner Elektronenspeicherring-Gesellschaft m.b.H. (BESSY).
* asynDriver is distributed subject to a Software License Agreement
* found in file LICENSE that is included with this distribution.
***********************************************************************/

/*RTEMS dependent file
 * Author: Shifu Xu
 * Date:   03NOV2005
change log:
   11/21/05  use only lower 32 bit of the timebase register in getTs()

 */
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>

#include <iocsh.h>
#include <epicsEvent.h>
#include <cantProceed.h>
#include <epicsExport.h>
#include <epicsInterrupt.h>
#include <devLib.h>

#include <rtems.h>
#include <bsp.h>
#include <bsp/VME.h>
#include <bsp/bspExt.h>
#include <bsp/vmeUniverse.h>

#include "sampler.h"

static void sysBusIntGen (int level, int vector);
typedef struct samplerOncePvt
{
    int vector;
    int level;
    double clockRateMhz;
    epicsEventId interruptEvent;
    unsigned long tsUserInterrupt;

} samplerOncePvt;

static unsigned long getTs (void)
{
    unsigned long low;
    __asm__ __volatile__ ("mftb %0":"=r" (low):);
    return low;
}

static void samplerIntService (void *pvt, unsigned long vector)
{
    samplerOncePvt *psamplerOncePvt = (samplerOncePvt *) pvt;
    psamplerOncePvt->tsUserInterrupt = getTs ();
    epicsEventSignal (psamplerOncePvt->interruptEvent);
}
static int getSample (void *pvt, delayMeasure * pdelayMeasure)
{
    unsigned long tsInterrupt;
    unsigned long tsUserInterrupt;
    unsigned long tsContextSwitch;
    int keys;
    samplerOncePvt *psamplerOncePvt = (samplerOncePvt *) pvt;
    keys = epicsInterruptLock ();
    tsInterrupt = getTs ();
    sysBusIntGen (psamplerOncePvt->level, psamplerOncePvt->vector);
    epicsInterruptUnlock (keys);
    epicsEventMustWait (psamplerOncePvt->interruptEvent);
    tsContextSwitch = getTs ();
    tsUserInterrupt = psamplerOncePvt->tsUserInterrupt;

    /* calculate */
    if (tsUserInterrupt >= tsInterrupt)
	pdelayMeasure->interruptLatency =
	    (unsigned long) (tsUserInterrupt - tsInterrupt) /
	    psamplerOncePvt->clockRateMhz;
    else
	pdelayMeasure->interruptLatency =
	    (unsigned long) (((unsigned long) 0xffffffff - tsInterrupt) +
			     tsUserInterrupt +
			     1) / psamplerOncePvt->clockRateMhz;

    if (tsContextSwitch >= tsUserInterrupt)
	pdelayMeasure->contextSwitch =
	    (unsigned long) (tsContextSwitch - tsUserInterrupt) /
	    psamplerOncePvt->clockRateMhz;
    else
	pdelayMeasure->contextSwitch =
	    (unsigned long) (((unsigned long) 0xffffffff - tsUserInterrupt) +
			     tsContextSwitch +
			     1) / psamplerOncePvt->clockRateMhz;

    if (tsContextSwitch >= tsInterrupt)
	pdelayMeasure->totalResponse =
	    (unsigned long) (tsContextSwitch - tsInterrupt) /
	    psamplerOncePvt->clockRateMhz;
    else
	pdelayMeasure->totalResponse =
	    (unsigned long) (((unsigned long) 0xffffffff - tsInterrupt) +
			     tsContextSwitch +
			     1) / psamplerOncePvt->clockRateMhz;

    pdelayMeasure->topToBottom = 0;
    return 0;
}

void platformSamplerInit (const char *name, int vector, int level,
			  double clockRateMhz)
{
    samplerOncePvt *psamplerOncePvt;
    psamplerOncePvt = callocMustSucceed (1, sizeof (samplerOncePvt), "name");
    psamplerOncePvt->level = level;
    psamplerOncePvt->vector = vector;
    psamplerOncePvt->clockRateMhz = clockRateMhz;
    psamplerOncePvt->interruptEvent = epicsEventCreate (epicsEventEmpty);
    psamplerOncePvt->tsUserInterrupt = 0;
    if (devConnectInterruptVME
	(vector, samplerIntService, (void *) psamplerOncePvt) != 0)
      {
	  epicsPrintf (" sampler intConnect failed");
	  return;
      }
    if (devEnableInterruptLevelVME (level) != 0)
      {
	  epicsPrintf (" sampler devEnableInterruptLevelVME  failed");
	  return;
      }
    psamplerManager->registerSampler (name, getSample, psamplerOncePvt);
}

void platformSamplerQuit (void *pvt)
{
    samplerOncePvt *psamplerOncePvt = (samplerOncePvt *) pvt;

    psamplerManager->removeSampler (pvt);
    free (psamplerOncePvt);
}

static const iocshArg initArg0 = { "platformSamplerInit", iocshArgString };
static const iocshArg initArg1 = { "interrupt vector", iocshArgInt };
static const iocshArg initArg2 = { "interrupt level", iocshArgInt };
static const iocshArg initArg3 = { "clock rate in Mhz", iocshArgDouble };

static const iocshArg *const initArgs[4] =
    { &initArg0, &initArg1, &initArg2, &initArg3 };
static void initCallFunc (const iocshArgBuf * args)
{
    platformSamplerInit (args[0].sval, args[1].ival, args[2].ival,
			 args[3].dval);
}

static const iocshFuncDef initFuncDef = { "platformSamplerInit", 4, initArgs };

static void sysBusIntGen (int level, int vector)
{
    vmeUniverseWriteReg (vector << 24, UNIV_REGOFF_VINT_STATID);
    vmeUniverseWriteReg (vmeUniverseReadReg (UNIV_REGOFF_VINT_EN) &
			 ~(1 << (24 + level)), UNIV_REGOFF_VINT_EN);
    vmeUniverseWriteReg (vmeUniverseReadReg (UNIV_REGOFF_VINT_EN) |
			 (1 << (24 + level)), UNIV_REGOFF_VINT_EN);
}

static void platformSamplerRegister (void)
{
    static int firstTime = 1;
    if (!firstTime)
	return;
    firstTime = 0;
    iocshRegister (&initFuncDef, initCallFunc);
}

epicsExportRegistrar (platformSamplerRegister);
